#include "fxlib.h"
#include <string.h>
#include "MonochromeLib.h"
#include "paint.h"

#define SIZE  1024
#define min(x,y)   (x>y?y:x)
#define max(x,y)   (x<y?y:x)
#define abs(x)	(x<0?-x:x)
#define sgn(x)   (x<0?-1:1)


void action(char *pict)
{
	int key, i;
	int cursorx = 64, cursory = 32;
	int select = 0, selectx, selecty;
	int coord = 0, coordx, coordy;
	int screen = 0;
	char screenmem[2*SIZE] = {0};
	int tempis = 0, tempx, tempy;
	char temp[SIZE] = {0};
	
	// initialization of the memory
	memcpy(&screenmem, pict, 2*SIZE);
	
	//main loop of the paint
	while(1) {
		
		//display
		ML_clear_vram();
		vramcpy(&screenmem[screen*SIZE], SIZE);
		display_cursor(cursorx, cursory);
		if(select) display_select(cursorx, cursory, selectx, selecty);
		if(coord) display_coord(cursorx+1, cursory+1);
		ML_display_vram();
		
		//getkey
		GetKey(&key);
		switch(key) {
			case KEY_CTRL_LEFT: {
				if(cursorx > 0) cursorx -= 1;
			}
			break;
			case KEY_CTRL_RIGHT: {
				if(cursorx < 127) cursorx += 1;
			}
			break;
			case KEY_CTRL_UP: {
				if(cursory > 0) cursory -= 1;
			}
			break;
			case KEY_CTRL_DOWN: {
				if(cursory < 63) cursory += 1;
			}
			break;
			case KEY_CTRL_F1: { //change screen
				screen = !screen;
			}
			break;
			case KEY_CTRL_F2: { //pixel
				//xor on 1 pixel
				screenmem[screen*SIZE+cursorx/8+16*cursory] ^= 128>>(cursorx&7);
			}
			break;
			case KEY_CTRL_F3: { //selection
				select = !select;
				if(select) {
					selectx = cursorx;
					selecty = cursory;
				}
			}
			break;
			case KEY_CTRL_F4: { //cut
				if(select) {
					select = !select;
					tempis = 1;
					tempx = abs(cursorx-selectx);
					tempy = abs(cursory-selecty);
					//todo
				}
			}
			break;
			case KEY_CTRL_F5: { //copy
				if(select) {
					select = !select;
					tempis = 1;
					tempx = abs(cursorx-selectx);
					tempy = abs(cursory-selecty);
					//todo
				}
			}
			break;
			case KEY_CTRL_F6: { //paste
				if(tempis) {
					//todo
				}
			}
			break;
			case KEY_CTRL_SHIFT: { //previous
				//todo
			}
			break;
			case KEY_CTRL_OPTN: { //coordinates
				coord = !coord;
			}
			break;
			case KEY_CTRL_EXIT: { //exit
				memcpy(pict, &screenmem, 2*SIZE);
				return;
			}
		}
	}
}


void display_cursor(int x, int y)
{
	ML_horizontal_line(y, max(x-2,0), min(x+2,127), ML_XOR);
	ML_vertical_line(x, max(y-2,0), min(y+2,63), ML_XOR);
	return;
}

void display_select(int x1, int y1, int x2, int y2)
{
	if(!(x1==x2 && y1==y2)) {
		if(x1==x2) ML_vertical_line(x1, y2, y1+3*sgn(y2-y1), ML_XOR);
		else if(y1==y2) ML_horizontal_line(y1, x2, x1+3*sgn(x2-x1), ML_XOR);
		else {
			ML_horizontal_line(y1, x2, x1+3*sgn(x2-x1), ML_XOR);
			ML_horizontal_line(y2, x2, x1, ML_XOR);
			if(y1>y2) {
				ML_vertical_line(x1, y2+1, y1+3*sgn(y2-y1), ML_XOR);
				ML_vertical_line(x2, y2+1, y1-1, ML_XOR);
			}
			else {
				ML_vertical_line(x1, y2-1, y1+3*sgn(y2-y1), ML_XOR);
				ML_vertical_line(x2, y2-1, y1+1, ML_XOR);
			}
		}
	}
	return;
}

void display_coord(int x, int y)
{
	int posy = 58, pos = 0, n = 0, i;
	char coord[7] = {0};
	
	for(i=x ; i ; i/=10) pos++;
	for(i=x ; i ; i/=10) {
		n++;
		coord[pos-n] = i%10+'0';
	}
	coord[pos] = ' ';
	n = 0;
	for(i=y ; i ; i/=10) pos++;
	for(i=y ; i ; i/=10) {
		n++;
		coord[pos+1-n] = i%10+'0';
	}
	coord[pos+1] = '\0';

	if(x<40 && y>35) posy = 1;
	PrintMini(1, posy, &coord, MINI_OVER);
	return;
}



int pictsize(char *pict, int size)
{
	int key, coord = 0;
	char screen1[SIZE] = {0}, screen2[SIZE] = {0};
	
	memcpy(&screen1, pict, SIZE);
	memcpy(&screen2, pict+SIZE, SIZE);
	
	do {
		//display
		ML_clear_vram();
		if(size<=1024) {
			vramcpy(&screen1, SIZE);
			ML_horizontal_line(size/16-1, 0, 127, ML_XOR);
		}
		else {
			vramcpy(&screen2, SIZE);
			ML_horizontal_line(size/16-65, 0, 127, ML_XOR);
		}
		if(coord) display_size(size);
		ML_display_vram();
		
		//getkey
		GetKey(&key);
		switch(key) {
			case KEY_CTRL_UP: {
				if(size>16) size -= 16;
			}
			break;
			case KEY_CTRL_DOWN: {
				if(size<2048) size += 16;
			}
			break;
			case KEY_CTRL_OPTN: {
				coord = !coord;
			}
			break;
			case KEY_CTRL_EXIT: {
				return 2*SIZE;
			}
		}
	}while(key!=KEY_CTRL_EXE);
	return size;
}

void display_size(int size)
{
	int posy = 58, pos = 0, n = 0, i;
	char coord[5] = {0};
	
	for(i=size ; i ; i/=10) pos++;
	for(i=size ; i ; i/=10) {
		n++;
		coord[pos-n] = i%10+'0';
	}
	coord[pos] = '\0';

	if((size-1)%1024>512) posy = 1;
	PrintMini(1, posy, &coord, MINI_OVER);
	return;
}


